/* SPDX-License-Identifier: AGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2023 Wojtek Porczyk <woju@hackerspace.pl>
 */

#include <Maqueen.h>
#include <Adafruit_Microbit.h>

#ifndef TUXGO_SPIN_DELAY
#   define TUXGO_SPIN_DELAY 340
#endif
#ifndef TUXGO_DRIVE_DELAY
#   define TUXGO_DRIVE_DELAY 500
#endif

Maqueen bot;
Adafruit_Microbit microbit;
NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);

void setup()
{
    Serial.begin(115200);
    Serial.println("init");
    bot.begin();

    microbit.BTLESerial.setLocalName("tuxgo");
    microbit.BTLESerial.begin();
    microbit.begin();
}

void loop()
{
    microbit.BTLESerial.poll();

    if (microbit.BTLESerial) { /* if is connected */
        if (microbit.BTLESerial.available()) {
            int c = microbit.BTLESerial.read();
            Serial.print("recv ");
            Serial.println(c);
            switch (c) {
            case 'H':
                Serial.println("backward");
                bot.setSpeed(50);
                bot.spinLeft();
                delay(TUXGO_SPIN_DELAY);
                bot.stop();
                Serial.println("F");
                microbit.BTLESerial.write('F');
                Serial.println("sent");
                break;

            case 'J':
                Serial.println("backward");
                bot.setSpeed(50);
                bot.backward();
                delay(TUXGO_DRIVE_DELAY);
                bot.stop();
                Serial.println("F");
                microbit.BTLESerial.write('F');
                Serial.println("sent");
                break;

            case 'K':
                Serial.println("forward");
                bot.setSpeed(50);
                bot.forward();
                delay(TUXGO_DRIVE_DELAY);
                bot.stop();
                Serial.println("F");
                microbit.BTLESerial.write('F');
                Serial.println("sent");
                break;

            case 'L':
                Serial.println("backward");
                bot.setSpeed(50);
                bot.spinRight();
                delay(TUXGO_SPIN_DELAY);
                bot.stop();
                Serial.println("F");
                microbit.BTLESerial.write('F');
                Serial.println("sent");
                break;

            case 'S':
                Serial.println("sonar");
                unsigned int range = sonar.ping_cm();
                microbit.BTLESerial.write(range < 256 ? range : 255);
                Serial.print(range);
                Serial.println(" cm");
                break;
            }
        }
    }
}
